# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import sqlite3


def run(job_input):
    db_connection = sqlite3.connect(
        "chinook.db"
    )  # if chinook.db file is not in your current directory, replace "chinook.db" with the path to your chinook.db file
    cursor = db_connection.cursor()
    cursor.execute("SELECT * FROM employees")
    job_input.send_tabular_data_for_ingestion(
        cursor,
        column_names=[column_info[0] for column_info in cursor.description],
        destination_table="backup_employees",
    )



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import logging

import tornado
from jupyterlab.labapp import LabApp

from ._version import __version__
from .config import VdkJupyterConfig
from .handlers import setup_handlers

log = logging.getLogger(__name__)


def _jupyter_labextension_paths():
    return [{"src": "labextension", "dest": "vdk-jupyterlab-extension"}]


def _jupyter_server_extension_points():
    return [{"module": "vdk_jupyterlab_extension"}]


def _load_jupyter_server_extension(server_app: LabApp):
    """Registers the API handler to receive HTTP requests from the frontend extension.

    Parameters
    ----------
    server_app: jupyterlab.labapp.LabApp
        JupyterLab application instance
    """
    tornado.log.enable_pretty_logging()
    tornado.log.app_log.setLevel(tornado.log.logging.DEBUG)

    vdk_config = VdkJupyterConfig(config=server_app.config)
    log.info(f"VDK Jupyter config: {vdk_config.config}")
    log.info(
        f"VDK Jupyter oauth2_authorization_url: {vdk_config.oauth2_authorization_url}"
    )
    log.info(f"app config: {server_app.config}")

    setup_handlers(server_app.web_app, vdk_config)
    name = "vdk_jupyterlab_extension"
    server_app.log.info(f"Registered {name} server extension")


# For backward compatibility with notebook server - useful for Binder/JupyterHub
load_jupyter_server_extension = _load_jupyter_server_extension



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import os

from traitlets import Unicode
from traitlets.config import Configurable


class VdkJupyterConfig(Configurable):
    oauth2_authorization_url = Unicode(
        os.environ.get("OAUTH2_AUTHORIZATION_URL", ""),
        config=True,
        help="The Oauth2 authorization URL. "
        "This is the URL used to start the authentication process."
        "Used to redirect to the authorization provider for user to login.",
    )
    oauth2_token_url = Unicode(
        os.environ.get("OAUTH2_TOKEN_URL", ""),
        config=True,
        help="The Oauth2 token URL. "
        "Used in the second phase of authentication process. "
        "Used to exchange authorization code with access token.",
    )
    oauth2_client_id = Unicode(
        os.environ.get("OAUTH2_CLIENT_ID", ""),
        config=True,
        help="The Oauth2 client ID. Note that client secret is not specified "
        "since we only support native app workflow with PKCE (RFC 7636)",
    )
    oauth2_redirect_url = Unicode(
        "http://127.0.0.1:8787/vdk-jupyterlab-extension/login2",
        config=True,
        help="The Oauth2 Redirect URL (or callback URL)."
        " This is the URL that authorization provider will redirect back the user to with the authorization code."
        " If empty automatically the request URL will be used.",
    )
    rest_api_url = Unicode(
        os.environ.get("REST_API_URL", ""),
        config=True,
        help="The VDK Control Service REST API URL",
    )



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import json
import os
import pathlib
import shlex
import shutil
import subprocess
import tempfile
from pathlib import Path

from vdk.internal.control.command_groups.job.create import JobCreate
from vdk.internal.control.command_groups.job.delete import JobDelete
from vdk.internal.control.command_groups.job.deploy_cli_impl import JobDeploy
from vdk.internal.control.command_groups.job.download_job import JobDownloadSource
from vdk.internal.control.utils import cli_utils
from vdk.internal.control.utils.output_printer import InMemoryTextPrinter
from vdk_jupyterlab_extension.convert_job import ConvertJobDirectoryProcessor
from vdk_jupyterlab_extension.convert_job import DirectoryArchiver
from vdk_jupyterlab_extension.jupyter_notebook import clear_notebook_outputs


class RestApiUrlConfiguration:
    @staticmethod
    def get_rest_api_url():
        try:
            rest_api_url = os.environ["REST_API_URL"]
        except Exception as e:
            raise ValueError(
                "What happened: Missing environment variable REST_API_URL.\n"
                "Why it happened: This is probably caused by a corrupt environment.\n"
                "Consequences: The current environment cannot work as it cannot connect to the VDK Control Service.\n"
                "Countermeasures: Please alert your support team; alternatively, try restarting your environment."
            )
        return rest_api_url


class VdkUI:
    """
    A single parent class containing all the individual VDK methods in it.
    """

    @staticmethod
    def run_job(path, arguments=None):
        """
        Execute `run job`.
        :param path: the directory where the job run will be performed
        :param arguments: the additional variables for the job run
        :return: response with status code.
        """
        if not os.path.exists(path):
            path = os.getcwd() + path
            if not os.path.exists(path):
                return {"message": f"Incorrect path! {path} does not exist!"}
        script_files = [
            x
            for x in Path(path).iterdir()
            if (
                x.name.lower().endswith(".ipynb")
                or x.name.lower().endswith(".py")
                or x.name.lower().endswith(".sql")
            )
        ]
        if len(script_files) == 0:
            return {"message": f"No steps were found in {path}!"}
        with open("vdk_logs.txt", "w+") as log_file:
            path = shlex.quote(path)
            cmd: list[str] = ["vdk", "run", f"{path}"]
            if arguments:
                arguments = shlex.quote(arguments)
                cmd.append("--arguments")
                cmd.append(f"{arguments}")
            process = subprocess.Popen(
                cmd,
                stdout=log_file,
                stderr=log_file,
                env=os.environ.copy(),
                shell=False,
            )
            process.wait()
            if process.returncode != 0:
                job_name = (
                    os.path.basename(path[:-1])
                    if path.endswith("/")
                    else os.path.basename(path)
                )
                error_file = os.path.join(
                    os.path.dirname(path), f".{job_name}_error.json"
                )
                if os.path.exists(error_file):
                    with open(error_file) as file:
                        # the json is generated in vdk-notebook plugin
                        # you can see /vdk-notebook/src/vdk/notebook-plugin.py
                        error = json.load(file)
                        return {"message": error["details"]}
            return {"message": process.returncode}

    @staticmethod
    def delete_job(name: str, team: str) -> str:
        """
        Execute `delete job`.
        :param name: the name of the data job that will be deleted
        :param team: the team of the data job that will be deleted
        :return: message that the job is deleted
        """
        cmd = JobDelete(RestApiUrlConfiguration.get_rest_api_url())
        cmd.delete_job(name, team)
        return f"Deleted the job with name {name} from {team} team. "

    @staticmethod
    def download_job(name: str, team: str, path: str) -> str:
        """
        Execute `download job`.
        :param name: the name of the data job that will be downloaded
        :param team: the team of the data job that will be downloaded
        :param path: the path to the directory where the job will be downloaded
        :return: message that the job is downloaded
        """
        cmd = JobDownloadSource(RestApiUrlConfiguration.get_rest_api_url())
        cmd.download(team, name, path)
        return f"Downloaded the job with name {name} to {path}. "

    @staticmethod
    def create_job(name: str, team: str, path: str) -> str:
        """
        Execute `create job`.
        :param name: the name of the data job that will be created
        :param team: the team of the data job that will be created
        :param path: the path to the directory where the job will be created
        :return: message that the job is created
        """
        jupyter_job_dir = os.path.abspath(
            os.path.join(os.path.dirname(__file__), "jupyter_sample_job")
        )
        rest_api_url = ""
        cloud = False
        error = ""
        try:
            rest_api_url = RestApiUrlConfiguration.get_rest_api_url()
            cli_utils.check_rest_api_url(rest_api_url)
            cloud = True
        except ValueError as e:
            error = str(e)
        cmd = JobCreate(rest_api_url)
        cmd.create_job(name, team, path, cloud, True, pathlib.Path(jupyter_job_dir))
        if cloud:
            result = f"Job with name {name} was created successfully!"
        else:
            result = (
                f"Job with name {name} was created only locally. "
                f"If you are not using the Control Service the next lines should not concern you! \n"
                f"We tried to create it in the cloud but come up to:"
                f"{error}"
                f""
            )

        return result

    @staticmethod
    def create_deployment(name: str, team: str, path: str, reason: str):
        """
        Execute `Deploy job`.
        :param name: the name of the data job that will be deployed
        :param team: the team of the data job that will be deployed
        :param path: the path to the job's directory
        :param reason: the reason of deployment
        :return: output string of the operation
        """
        with tempfile.TemporaryDirectory() as temp_dir:
            if not os.path.exists(path):
                raise NotADirectoryError(
                    f"The provided path '{path}' is not a valid directory."
                )

            # Copy the contents of the original directory to the temporary directory
            for item in os.listdir(path):
                source_item = os.path.join(path, item)
                destination_item = os.path.join(temp_dir, item)
                if os.path.isdir(source_item):
                    shutil.copytree(source_item, destination_item)
                else:
                    shutil.copy2(source_item, destination_item)

            # Clear outputs for all notebooks in the temporary directory
            for dir_path, _, filenames in os.walk(temp_dir):
                for filename in filenames:
                    if filename.endswith(".ipynb"):
                        notebook_path = os.path.join(dir_path, filename)
                        clear_notebook_outputs(notebook_path)

            printer = InMemoryTextPrinter()

            try:
                cmd = JobDeploy(RestApiUrlConfiguration.get_rest_api_url(), printer)
                cmd.create(
                    name=name,
                    team=team,
                    job_path=temp_dir,
                    reason=reason,
                    vdk_version=None,
                    enabled=True,
                )
                return (
                    f"Request to deploy job with name {name} and team {team} is sent successfully!"
                    f"It would take a few minutes for the Data Job to be deployed in the server."
                    f"If notified_on_job_deploy option in config.ini is configured then "
                    f"notification will be sent on successful deploy or in case of an error.\n\n"
                    f"Deployment information:\n {printer.get_memory().strip()}"
                )

            finally:
                # Temporary directory will be automatically cleaned up
                pass

    @staticmethod
    def get_notebook_info(cell_id: str, pr_path: str):
        """
        Return information about the notebook that includes a cell with a given id
        :param cell_id: the id of the cell
        :param pr_path: the path to the parent directory of the notebook
        :return: path of the notebook, index of the cell
             if the specified parent directory does not exist, an empty dictionary is returned.
        """
        if not os.path.exists(pr_path):
            pr_path = os.getcwd() + pr_path
            if not os.path.exists(pr_path):
                return {"path": "", "cellIndex": ""}
        notebook_files = [
            x
            for x in pathlib.Path(pr_path).iterdir()
            if (x.name.lower().endswith(".ipynb"))
        ]
        for notebook_file in notebook_files:
            with open(notebook_file) as f:
                notebook_json = json.load(f)
                for index, jupyter_cell in enumerate(notebook_json["cells"]):
                    if jupyter_cell["id"] == cell_id:
                        notebook_path = str(notebook_file).replace(os.getcwd(), "")
                        return {
                            "path": str(notebook_path),
                            "cellIndex": str(index),
                        }
        return {"path": "", "cellIndex": ""}

    @staticmethod
    def get_vdk_tagged_cell_indices(notebook_path: str):
        """
        Return the indices of the notebook cells that are tagged with 'vdk'
        :param notebook_path: Path to the notebook file.
        :return: A list containing the indices of the cells tagged with 'vdk'.
             If the specified notebook path does not exist, an empty list is returned.
        """
        if not os.path.exists(notebook_path):
            job_notebook = os.getcwd() + notebook_path
            if not os.path.exists(job_notebook):
                return []
        with open(notebook_path) as f:
            notebook_json = json.load(f)
            vdk_cells = []
            for index, jupyter_cell in enumerate(notebook_json["cells"]):
                if "vdk" in jupyter_cell["metadata"].get("tags", {}):
                    vdk_cells.append(index)
            return vdk_cells

    @staticmethod
    def convert_job(job_dir: str):
        """
        Transforms the job in the specified directory by archiving it, processing the Python and SQL files,
        and returning a processed code structure.
        :param job_dir: Path to the directory of the job to be transformed.
        :return: The processed code structure.
        """
        job_dir = Path(job_dir)
        archiver = DirectoryArchiver(job_dir)
        processor = ConvertJobDirectoryProcessor(job_dir)
        archiver.archive_folder()
        processor.process_files()
        message = {
            "code_structure": processor.get_code_structure(),
            "removed_files": processor.get_removed_files(),
        }
        processor.cleanup()
        return message



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import shutil
import tempfile
from unittest.mock import ANY
from unittest.mock import Mock
from unittest.mock import patch

import pytest
from vdk_jupyterlab_extension.vdk_ui import RestApiUrlConfiguration
from vdk_jupyterlab_extension.vdk_ui import VdkUI


@pytest.fixture
def temp_directory():
    with tempfile.TemporaryDirectory() as temp_dir:
        yield temp_dir


@patch("vdk_jupyterlab_extension.vdk_ui.clear_notebook_outputs")
@patch("vdk_jupyterlab_extension.vdk_ui.JobDeploy")
@patch("vdk_jupyterlab_extension.vdk_ui.InMemoryTextPrinter")
def test_create_deployment(
    mock_printer, mock_job_deploy, mock_clear_notebook_outputs, temp_directory
):
    path = temp_directory
    source_dir = os.path.abspath(
        os.path.join(os.path.dirname(__file__), "data/ingest-notebook")
    )
    for item in os.listdir(source_dir):
        source_item = os.path.join(source_dir, item)
        destination_item = os.path.join(path, item)
        if os.path.isfile(source_item):
            shutil.copy2(source_item, destination_item)

    with patch.object(
        RestApiUrlConfiguration, "get_rest_api_url", return_value="http://dummy-api-url"
    ):
        mock_clear_notebook_outputs.return_value = None
        mock_cmd_instance = Mock()
        mock_job_deploy.return_value = mock_cmd_instance
        mock_printer_instance = Mock()
        mock_printer.return_value = mock_printer_instance

        mock_cmd_instance.create.return_value = None

        name = "TestJob"
        team = "TestTeam"
        reason = "Testing"

        output = VdkUI.create_deployment(name, team, path, reason)

        assert "Request to deploy job" in output
        assert "sent successfully" in output
        assert "Deployment information:" in output

        assert mock_clear_notebook_outputs.called

        mock_job_deploy.assert_called_once_with(
            "http://dummy-api-url", mock_printer_instance
        )
        mock_cmd_instance.create.assert_called_once_with(
            name=name,
            team=team,
            job_path=ANY,
            reason=reason,
            vdk_version=None,
            enabled=True,
        )


@patch("vdk_jupyterlab_extension.vdk_ui.clear_notebook_outputs")
@patch("vdk_jupyterlab_extension.vdk_ui.JobDeploy")
@patch("vdk_jupyterlab_extension.vdk_ui.InMemoryTextPrinter")
def test_create_deployment_with_invalid_path(
    mock_printer, mock_job_deploy, mock_clear_notebook_outputs, temp_directory
):
    path = "/invalid/path"
    with patch.object(
        RestApiUrlConfiguration, "get_rest_api_url", return_value="http://dummy-api-url"
    ):
        mock_clear_notebook_outputs.return_value = None
        mock_cmd_instance = Mock()
        mock_job_deploy.return_value = mock_cmd_instance
        mock_printer_instance = Mock()
        mock_printer.return_value = mock_printer_instance

        mock_cmd_instance.create.return_value = None

        name = "TestJob"
        team = "TestTeam"
        reason = "Testing"

        with pytest.raises(NotADirectoryError):
            VdkUI.create_deployment(name, team, path, reason)

        # Ensure clear_notebook_outputs and other calls were not made
        mock_clear_notebook_outputs.assert_not_called()
        mock_job_deploy.assert_not_called()
        mock_printer.assert_not_called()
        mock_cmd_instance.create.assert_not_called()



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import nbformat
from nbconvert.preprocessors import ClearOutputPreprocessor


def clear_notebook_outputs(notebook_path: str):
    """
    Clear the outputs of a Jupyter notebook.
    :param notebook_path: Path to the Jupyter notebook.
    """
    with open(notebook_path, encoding="utf-8") as f:
        notebook = nbformat.read(f, as_version=4)
        clearer = ClearOutputPreprocessor()
        clearer.preprocess(notebook, {})

    with open(notebook_path, "w", encoding="utf-8") as f:
        nbformat.write(notebook, f)



# Copyright 2023-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import pathlib

from vdk.internal.builtin_plugins.config.job_config import JobConfig


class JobDataLoader:
    """
      The JobDataLoader class is used for retrieving job-specific data from a provided directory.
    Attributes:
        _path (Path): A Path object representing the path to the job's directory.
        _config (JobConfig): A JobConfig object used to access job configuration data.
    Methods:
        get_job_path(): Returns the absolute path of the job directory as a string.
        get_job_name(): Returns the name of the job.
        get_team_name(): Returns the team name associated with the job.
    """

    def __init__(self, working_directory: str):
        self._path = pathlib.Path(os.getcwd() + "/" + working_directory)
        if not os.path.isdir(self._path):
            raise ValueError(f"{self._path} is not a valid directory.")
        self._config = JobConfig(self._path)

    def get_job_path(self) -> str:
        return str(self._path)

    def get_job_name(self) -> str:
        return os.path.basename(self._path)

    def get_team_name(self) -> str:
        if self._config:
            return self._config.get_team()
        return ""



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import os
from unittest import mock
from unittest.mock import MagicMock
from urllib.parse import parse_qs
from urllib.parse import urlparse

from tornado import httputil
from tornado.web import Application
from vdk.plugin.control_api_auth.base_auth import BaseAuth
from vdk_jupyterlab_extension.handlers import OAuth2Handler


def test_fix_redirect_uri():
    assert (
        OAuth2Handler._fix_localhost("http://localhost?foo=bar") == "http://127.0.0.1"
    )
    assert (
        OAuth2Handler._fix_localhost("http://localhost:8888?foo=bar")
        == "http://127.0.0.1:8888"
    )
    assert (
        OAuth2Handler._fix_localhost("http://something?foo=bar") == "http://something"
    )
    assert (
        OAuth2Handler._fix_localhost("http://something:9999?foo=bar")
        == "http://something:9999"
    )
    assert (
        OAuth2Handler._fix_localhost("http://something:9999") == "http://something:9999"
    )


import json
import httpretty
import pytest
from vdk_jupyterlab_extension import VdkJupyterConfig


@pytest.fixture(scope="module")
def vdk_config():
    return VdkJupyterConfig(
        oauth2_authorization_url="https://example.com/auth",
        oauth2_token_url="https://example.com/token",
        oauth2_client_id="sample_client_id",
        oauth2_redirect_url="http://my.app.com/redirect",
    )


@pytest.fixture
def oauth2_handler(vdk_config):
    request = httputil.HTTPServerRequest(
        uri="http://my.app.com/login", method="GET", connection=MagicMock()
    )
    handler = OAuth2Handler(Application(), request, vdk_config=vdk_config)
    handler.finish = MagicMock()
    return handler


@httpretty.activate
def test_get_without_code(oauth2_handler):
    # Arrange
    httpretty.register_uri(
        httpretty.GET,
        "https://example.com/auth",
        body="redirect_to_auth_server",
    )

    oauth2_handler.request.arguments = {}

    # Act
    oauth2_handler.get()

    # Assert
    mock_finish: MagicMock = oauth2_handler.finish

    auth_url = urlparse(mock_finish.call_args[0][0])
    assert auth_url.netloc == "example.com"
    assert auth_url.path == "/auth"
    assert auth_url.scheme == "https"

    query_params = parse_qs(auth_url.query)
    assert query_params.get("response_type") == ["code"]
    assert query_params.get("client_id") == ["sample_client_id"]
    assert query_params.get("code_challenge")


@httpretty.activate
def test_get_with_code(oauth2_handler: OAuth2Handler, tmpdir):
    with mock.patch.dict(
        os.environ,
        {"VDK_BASE_CONFIG_FOLDER": str(tmpdir)},
    ):
        # Arrange
        oauth2_handler.application.settings["code_verifier"] = "random"
        mock_token_response = {
            "access_token": "sample_token",
            "expires_in": 3600,
        }
        httpretty.register_uri(
            httpretty.POST,
            "https://example.com/token",
            body=json.dumps(mock_token_response),
        )

        # Mock the request to contain a code
        oauth2_handler.request.arguments = {"code": ["sample_code"]}

        # Act
        oauth2_handler.get()

        # Assert
        # The access token must be safely stored in VDK auth storage
        auth = BaseAuth()
        assert auth.read_access_token() == "sample_token"



# Copyright 2023-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
from enum import Enum


class VdkOption(Enum):
    NAME = "jobName"
    TEAM = "jobTeam"
    PATH = "jobPath"
    ARGUMENTS = "jobArguments"
    DEPLOYMENT_REASON = "deploymentReason"



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import pytest

pytest_plugins = ("jupyter_server.pytest_plugin",)


@pytest.fixture
def jp_server_config(jp_server_config):
    return {"ServerApp": {"jpserver_extensions": {"vdk_jupyterlab_extension": True}}}



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import json
import logging
import time
from urllib.parse import urlparse
from urllib.parse import urlunparse

import requests
from jupyter_server.base.handlers import APIHandler
from requests.auth import HTTPBasicAuth
from requests_oauthlib import OAuth2Session
from vdk.plugin.control_api_auth.auth_request_values import AuthRequestValues
from vdk.plugin.control_api_auth.autorization_code_auth import generate_pkce_codes
from vdk.plugin.control_api_auth.base_auth import BaseAuth
from vdk_jupyterlab_extension import VdkJupyterConfig

log = logging.getLogger(__name__)


class OAuth2Handler(APIHandler):
    def initialize(self, vdk_config: VdkJupyterConfig):
        log.info(f"VDK config: {vdk_config.__dict__}")

        self._authorization_url = vdk_config.oauth2_authorization_url
        self._access_token_url = vdk_config.oauth2_token_url
        self._client_id = vdk_config.oauth2_client_id
        self._redirect_url = vdk_config.oauth2_redirect_url

        log.info(f"Authorization URL: {self._authorization_url}")
        log.info(f"Access Token URL: {self._access_token_url}")
        # log.info(f"client_id: {self._client_id}")

        # No client secret. We use only native app workflow with PKCE (RFC 7636)

    @staticmethod
    def _fix_localhost(uri: str):
        """
        This is added for local testing. Oauthorization Providers generally allow 127.0.0.1 to be registered as redirect URL
        so we change localhost to 127.0.0.1
        :param uri:
        :return:
        """
        parsed_uri = urlparse(uri)

        if parsed_uri.hostname == "localhost":
            netloc = parsed_uri.netloc.replace("localhost", "127.0.0.1")
            modified_uri = parsed_uri._replace(netloc=netloc, query="")
            return urlunparse(modified_uri)
        else:
            modified_uri = parsed_uri._replace(query="")
            return urlunparse(modified_uri)

    def get(self):
        # TODO: this is duplicating a lot of the code in vdk-control-api-auth
        # https://github.com/vmware/versatile-data-kit/tree/main/projects/vdk-plugins/vdk-control-api-auth
        # But that module is written with focus on CLI usage a bit making it harder to reuse
        # and it needs to be refactored first.
        redirect_url = self._redirect_url
        if not redirect_url:
            redirect_url = self.request.full_url()
        redirect_url = self._fix_localhost(redirect_url)

        log.info(f"redirect uri is {redirect_url}")

        if self.get_argument("code", None):
            log.info(
                "Authorization code received. Will generate access token using authorization code."
            )
            tokens = self._exchange_auth_code_for_access_token(redirect_url)
            log.info(f"Got tokens data: {tokens}")  # TODO: remove this
            self._persist_tokens_data(tokens)
        else:
            log.info(f"Authorization URL is: {self._authorization_url}")
            full_authorization_url = self._prepare_authorization_code_request_url(
                redirect_url
            )
            self.finish(full_authorization_url)

    def _persist_tokens_data(self, tokens):
        auth = BaseAuth()
        auth.update_oauth2_authorization_url(self._access_token_url)
        auth.update_client_id(self._client_id)
        auth.update_access_token(tokens.get(AuthRequestValues.ACCESS_TOKEN_KEY.value))
        auth.update_access_token_expiration_time(
            time.time() + int(tokens[AuthRequestValues.EXPIRATION_TIME_KEY.value])
        )
        if AuthRequestValues.REFRESH_TOKEN_GRANT_TYPE in tokens:
            auth.update_refresh_token(
                tokens.get(AuthRequestValues.REFRESH_TOKEN_GRANT_TYPE)
            )

    def _prepare_authorization_code_request_url(self, redirect_uri):
        (code_verifier, code_challenge, code_challenge_method) = generate_pkce_codes()
        self.application.settings["code_verifier"] = code_verifier
        oauth = OAuth2Session(client_id=self._client_id, redirect_uri=redirect_uri)
        full_authorization_url = oauth.authorization_url(
            self._authorization_url,
            state="requested",
            prompt=AuthRequestValues.LOGIN_PROMPT.value,
            code_challenge=code_challenge,
            code_challenge_method=code_challenge_method,
        )[0]
        return full_authorization_url

    def _exchange_auth_code_for_access_token(self, redirect_uri) -> dict:
        code = self.get_argument("code")
        headers = {
            AuthRequestValues.CONTENT_TYPE_HEADER.value: AuthRequestValues.CONTENT_TYPE_URLENCODED.value,
        }
        code_verifier = self.application.settings["code_verifier"]

        data = (
            f"code={code}&"
            + f"grant_type=authorization_code&"
            + f"code_verifier={code_verifier}&"
            f"redirect_uri={redirect_uri}"
        )
        basic_auth = HTTPBasicAuth(self._client_id, "")
        try:
            # TODO : this should be async io
            response = requests.post(
                self._access_token_url, data=data, headers=headers, auth=basic_auth
            )
            if response.status_code >= 400:
                log.error(
                    f"Request to {self._access_token_url} with data {data} returned {response.status_code}\n"
                    rf"Reason: {response.reason}\dn"
                    f"Response content: {response.content}\n"
                    f"Response headers: {response.headers}"
                )

            json_data = json.loads(response.text)
            return json_data
        except Exception as e:
            log.exception(e)



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import glob
import os
import re
import shutil


def validate_dir(dir_path):
    if not os.path.isdir(dir_path):
        raise ValueError(f"{dir_path} is not a valid directory.")
    return True


class DirectoryArchiver:
    """
    The DirectoryArchiver class provides functionality to create an archive of a specified directory.
    Attributes:
        _dir_path (str): The path to the directory that will be archived.
    Methods:
        get_parent_dir(): Returns the parent directory of _dir_path.
        _get_archive_name(): Generates and returns the name of the archive file.
        archive_folder(): Creates a zip archive of the directory.
    """

    def __init__(self, dir_path):
        validate_dir(dir_path)
        self._dir_path = dir_path

    def get_parent_dir(self) -> str:
        return os.path.dirname(self._dir_path)

    def _get_archive_name(self) -> str:
        dir_name = os.path.basename(self._dir_path)
        parent_dir = self.get_parent_dir()
        return os.path.join(parent_dir, f"{dir_name}_archive")

    def archive_folder(self):
        shutil.make_archive(self._get_archive_name(), "zip", self._dir_path)


class ConvertJobDirectoryProcessor:
    """
    The ConvertJobDirectoryProcessor class provides functionality to process a directory,
    archiving it, and storing specific file contents into a structured list.
    Attributes:
        _dir_path (str): The path to the directory that will be processed.
        _code_structure (list): A list to store the processed content of specific files.
        _removed_files (list): A list to store the names of files that have been removed.
    Methods:
        process_files(): Processes files in the directory, saving and removing specific files' content.
        cleanup(): Removes any remaining files that were supposed to be removed.
        get_code_structure() -> list: Returns the structured list of processed file contents.
        get_removed_files() -> list: Returns the list of removed file names.
    """

    def __init__(self, dir_path):
        validate_dir(dir_path)
        self._dir_path = dir_path
        self._code_structure = []
        self._removed_files = []

    def process_files(self):
        all_files = sorted(glob.glob(os.path.join(self._dir_path, "*")))

        for file in all_files:
            if file.endswith(".sql"):
                with open(file) as f:
                    content = f.read()
                self._code_structure.append(f'job_input.execute_query("""{content}""")')
                self._removed_files.append(os.path.basename(file))
                os.remove(file)
            elif file.endswith(".py"):
                with open(file) as f:
                    content = f.read()

                if re.search(r"def run\(job_input", content):
                    self._code_structure.append(content)
                    self._removed_files.append(os.path.basename(file))
                    os.remove(file)

    def cleanup(self):
        for file_name in self._removed_files:
            file_path = os.path.join(self._dir_path, file_name)
            if os.path.exists(file_path):
                os.remove(file_path)

    def get_code_structure(self):
        return self._code_structure

    def get_removed_files(self):
        return self._removed_files



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import json
import logging
import os

import tornado
from jupyter_server.base.handlers import APIHandler
from jupyter_server.utils import url_path_join

from . import VdkJupyterConfig
from .job_data import JobDataLoader
from .oauth2 import OAuth2Handler
from .vdk_options.vdk_options import VdkOption
from .vdk_ui import VdkUI

log = logging.getLogger(__name__)


class LoadJobDataHandler(APIHandler):
    """
    Class responsible for handling POST request for retrieving data(full path, job's name and team)
     about job of directory
     Response: return a json formatted str providing the data about the job
    """

    @tornado.web.authenticated
    def post(self):
        working_directory = json.loads(self.get_json_body())[VdkOption.PATH.value]
        try:
            data = JobDataLoader(working_directory)
            self.finish(
                json.dumps(
                    {
                        VdkOption.PATH.value: data.get_job_path(),
                        VdkOption.NAME.value: data.get_job_name(),
                        VdkOption.TEAM.value: data.get_team_name(),
                    }
                )
            )
        except Exception as e:
            log.debug(
                f"Failed to load job information from config.ini with error: {e}."
            )
            self.finish(
                json.dumps(
                    {
                        VdkOption.PATH.value: "",
                        VdkOption.NAME.value: "",
                        VdkOption.TEAM.value: "",
                    }
                )
            )


class RunJobHandler(APIHandler):
    """
    Class responsible for handling POST request for running a Data Job given its path and arguments to run with
    Response: return a json formatted str including:
     ::error field with error message if an error exists
     ::message field with status of the VDK operation
    """

    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()
        run_result = VdkUI.run_job(
            input_data[VdkOption.PATH.value],
            input_data[VdkOption.ARGUMENTS.value],
        )
        self.finish(json.dumps(run_result))


class DeleteJobHandler(APIHandler):
    """
    Class responsible for handling POST request for deleting a Data Job given its name, team and Rest API URL
    Response: return a json formatted str including:
        ::error field with error message if an error exists
        ::message field with status of the Vdk operation
    """

    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()
        try:
            status = VdkUI.delete_job(
                input_data[VdkOption.NAME.value], input_data[VdkOption.TEAM.value]
            )
            self.finish(json.dumps({"message": f"{status}", "error": ""}))
        except Exception as e:
            self.finish(json.dumps({"message": f"{e}", "error": "true"}))


class DownloadJobHandler(APIHandler):
    """
    Class responsible for handling POST request for downloading a Data Job given its name, team,
    Rest API URL, and the path to where the job will be downloaded
    Response: return a json formatted str including:
        ::error field with error message if an error exists
        ::message field with status of the Vdk operation
    """

    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()
        try:
            status = VdkUI.download_job(
                input_data[VdkOption.NAME.value],
                input_data[VdkOption.TEAM.value],
                input_data[VdkOption.PATH.value],
            )
            self.finish(json.dumps({"message": f"{status}", "error": ""}))
        except Exception as e:
            self.finish(json.dumps({"message": f"{e}", "error": "true"}))


class ConvertJobHandler(APIHandler):
    """
    Class responsible for handling POST request for transforming a directory type Data job(with .py and .sql files)
    to a notebook type data job
    """

    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()
        try:
            message = json.dumps(VdkUI.convert_job(input_data[VdkOption.PATH.value]))
            self.finish(json.dumps({"message": f"{message}", "error": ""}))
        except Exception as e:
            self.finish(json.dumps({"message": f"{e}", "error": "true"}))


class CreateJobHandler(APIHandler):
    """
    Class responsible for handling POST request for creating a Data Job given its name, team,
    flags whether it will be created locally or in the cloud, path to where job will be created (if local),
    Rest API URL (if cloud)
    Response: return a json formatted str including:
        ::error field with error message if an error exists
        ::message field with status of the Vdk operation
    """

    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()
        try:
            status = VdkUI.create_job(
                input_data[VdkOption.NAME.value],
                input_data[VdkOption.TEAM.value],
                input_data[VdkOption.PATH.value],
            )
            self.finish(json.dumps({"message": f"{status}", "error": ""}))
        except Exception as e:
            self.finish(json.dumps({"message": f"{e}", "error": "true"}))


class CreateDeploymentHandler(APIHandler):
    """
    Class responsible for handling POST request for creating a deployment of  Data Job given its name, team, path,
    Rest API URL, deployment reason and flag whether it is enabled (that will basically un-pause the job)
    Response: return a json formatted str including:
        ::error field with error message if an error exists
        ::message field with status of the Vdk operation
    """

    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()
        try:
            status = VdkUI.create_deployment(
                input_data[VdkOption.NAME.value],
                input_data[VdkOption.TEAM.value],
                input_data[VdkOption.PATH.value],
                input_data[VdkOption.DEPLOYMENT_REASON.value],
            )
            self.finish(json.dumps({"message": f"{status}", "error": ""}))
        except Exception as e:
            self.finish(json.dumps({"message": f"{e}", "error": "true"}))


class GetNotebookInfoHandler(APIHandler):
    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()
        notebook_info = VdkUI.get_notebook_info(
            input_data["cellId"], input_data[VdkOption.PATH.value]
        )
        self.finish(json.dumps(notebook_info))


class GetVdkCellIndicesHandler(APIHandler):
    @tornado.web.authenticated
    def post(self):
        input_data = self.get_json_body()
        vdk_indices = VdkUI.get_vdk_tagged_cell_indices(input_data["nbPath"])
        self.finish(json.dumps(vdk_indices))


class GetServerPathHandler(APIHandler):
    @tornado.web.authenticated
    def get(self):
        self.finish(json.dumps(os.getcwd()))


def setup_handlers(web_app, vdk_config: VdkJupyterConfig):
    host_pattern = ".*$"
    base_url = web_app.settings["base_url"]

    def add_handler(handler, endpoint, args=None):
        job_route_pattern = url_path_join(
            base_url, "vdk-jupyterlab-extension", endpoint
        )
        if args is None:
            args = {}
        job_handlers = [(job_route_pattern, handler, args)]
        log.info(f"Job handlers: {job_handlers}")
        web_app.add_handlers(host_pattern, job_handlers)

    add_handler(OAuth2Handler, "login", {"vdk_config": vdk_config})
    add_handler(RunJobHandler, "run")
    add_handler(DeleteJobHandler, "delete")
    add_handler(DownloadJobHandler, "download")
    add_handler(ConvertJobHandler, "convertJobToNotebook")
    add_handler(CreateJobHandler, "create")
    add_handler(LoadJobDataHandler, "job")
    add_handler(CreateDeploymentHandler, "deploy")
    add_handler(GetNotebookInfoHandler, "notebook")
    add_handler(GetVdkCellIndicesHandler, "vdkCellIndices")
    add_handler(GetServerPathHandler, "serverPath")



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import json
import shutil
import tempfile
from pathlib import Path

import pytest
from vdk_jupyterlab_extension.jupyter_notebook import clear_notebook_outputs


@pytest.fixture(scope="function", autouse=True)
def create_temporary_directory():
    temp_dir = tempfile.TemporaryDirectory()
    yield temp_dir.name
    temp_dir.cleanup()


@pytest.mark.parametrize("job_dir", ["ingest-notebook"])
def test_notebook_output_cleaner(job_dir, create_temporary_directory):
    base_dir = Path(__file__).parent / "data"
    dir_path = base_dir / job_dir
    temp_dir = create_temporary_directory

    for notebook_path in dir_path.glob("*.ipynb"):
        temp_notebook_path = Path(temp_dir) / notebook_path.name
        shutil.copy2(notebook_path, temp_notebook_path)
        clear_notebook_outputs(str(temp_notebook_path))

    for notebook_path in dir_path.glob("*.ipynb"):
        temp_notebook_path = Path(temp_dir) / notebook_path.name
        with temp_notebook_path.open("r") as f:
            content = json.load(f)
        for cell in content["cells"]:
            if cell["cell_type"] == "code":
                assert cell["execution_count"] is None
                assert not cell["outputs"]



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import shutil
import tempfile
import unittest

from vdk_jupyterlab_extension.convert_job import ConvertJobDirectoryProcessor


class TestConvertJobDirectoryProcessor(unittest.TestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()
        self.sql_content = "SELECT * FROM table"
        self.py_content_run = """
        def run(job_input: IJobInput):
            print("Hello, World!")
        """
        self.py_content_without_run = """
        def hello():
            print("Hello, World!")
        """

        with open(os.path.join(self.temp_dir, "test.sql"), "w") as f:
            f.write(self.sql_content)
        with open(os.path.join(self.temp_dir, "test_run.py"), "w") as f:
            f.write(self.py_content_run)
        with open(os.path.join(self.temp_dir, "test_without_run.py"), "w") as f:
            f.write(self.py_content_without_run)
        with open(os.path.join(self.temp_dir, "config.ini"), "w") as f:
            pass

        self.processor = ConvertJobDirectoryProcessor(self.temp_dir)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_process_files(self):
        self.processor.process_files()
        expected_code_structure = [
            f'job_input.execute_query("""{self.sql_content}""")',
            self.py_content_run,
        ]
        self.assertEqual(self.processor.get_code_structure(), expected_code_structure)
        self.assertFalse(os.path.exists(os.path.join(self.temp_dir, "test.sql")))
        self.assertFalse(os.path.exists(os.path.join(self.temp_dir, "test_run.py")))
        self.assertTrue(
            os.path.exists(os.path.join(self.temp_dir, "test_without_run.py"))
        )
        self.assertTrue(os.path.exists(os.path.join(self.temp_dir, "config.ini")))

        expected_removed_files = ["test.sql", "test_run.py"]
        self.assertEqual(
            set(self.processor.get_removed_files()), set(expected_removed_files)
        )

    def test_cleanup(self):
        self.processor.process_files()
        self.processor.cleanup()
        self.assertFalse(os.path.exists(os.path.join(self.temp_dir, "test.sql")))
        self.assertFalse(os.path.exists(os.path.join(self.temp_dir, "test_run.py")))

    def test_get_code_structure(self):
        self.processor.process_files()
        expected_code_structure = [
            f'job_input.execute_query("""{self.sql_content}""")',
            self.py_content_run,
        ]
        self.assertEqual(self.processor.get_code_structure(), expected_code_structure)

    def test_get_removed_files(self):
        self.processor.process_files()
        expected_removed_files = ["test.sql", "test_run.py"]
        self.assertEqual(
            set(self.processor.get_removed_files()), set(expected_removed_files)
        )



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import shutil
import tempfile
import unittest

from vdk_jupyterlab_extension.convert_job import DirectoryArchiver


class TestDirectoryArchiver(unittest.TestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()
        self.archiver = DirectoryArchiver(self.temp_dir)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_get_parent_dir(self):
        self.assertEqual(self.archiver.get_parent_dir(), os.path.dirname(self.temp_dir))

    def test_get_archive_name(self):
        dir_name = os.path.basename(self.temp_dir)
        parent_dir = os.path.dirname(self.temp_dir)
        expected_archive_name = os.path.join(parent_dir, f"{dir_name}_archive")
        self.assertEqual(self.archiver._get_archive_name(), expected_archive_name)

    def test_archive_folder(self):
        self.archiver.archive_folder()
        archive_name = self.archiver._get_archive_name()
        self.assertTrue(os.path.exists(f"{archive_name}.zip"))
